from .base import RawReader, TextDumper


class TXT(RawReader, TextDumper):
    ext = 'txt', 'rc'

    def load(self, raw):
        return raw

    def parse(self, data):
        for line in data.split('\n'):
            yield line.rstrip()




from .csv import CSV
from .ini import INI
from .json import JSON
from .pickle import PICKLE
from .sqlite import SQLITE
from .toml import TOML
from .txt import TXT
from .yaml import YAML
from .bin import BIN, OCT, DEC, HEX
from .cfg import CFG
from .html import HTML

Plugins = (
    CSV,
    INI,
    JSON,
    PICKLE,
    SQLITE,
    TOML,
    TXT,
    YAML,
    BIN,
    OCT,
    DEC,
    HEX,
    CFG,
    HTML,
)




from os import listdir

from check_tools import capture

import caty


def test_static():
    for name in listdir('input'):
        if name.startswith('.'): continue
        ext = name.split('.')[1]
        print(name)
        exp = open(f'output/{name}').read()
        with capture(caty.main, f'input/{name}', ext) as (out, err):
            assert out==exp



from .base import RawReader, DataDumper


class PICKLE(RawReader, DataDumper):
    ext = 'pickle', 'pkl'
    mode = 'rb'

    def load(self, raw):
        from pickle import loads
        return loads(raw)



from tempfile import NamedTemporaryFile
from os.path import isfile, expanduser


class Reader:
    mode = 'r'

    def load(self, raw):
        raise NotImplemented

    def load_file(self, path):
        raise NotImplemented

    def parse(self, data):
        return data

    def read(self, path):
        raw = self.load_file(path)
        data = self.parse(raw)
        return data


class RawReader(Reader):
    def load_file(self, path):
        with open(path, self.mode) as inp:
            raw = inp.read()
        return self.load(raw)


class FileReader(Reader):
    def load(self, raw):
        with NamedTemporaryFile(delete=False) as tmp:
            tmp.write(raw)
        return self.load_file(tmp.name)


class Dumper:
    def dump(self, data):
        raise NotImplemented


class DataDumper(Dumper):
    @property
    def conf(self):
        from . import INI, JSON, TOML, YAML
        conf = dict()
        try:
            for reader in (INI, JSON, TOML, YAML):
                for ext in reader.ext:
                    confil = expanduser(f'~/.config/caty.{ext}')
                    if isfile(confil):
                        conf = reader().read(confil)
                        raise StopIteration
        except StopIteration:
            pass
        return conf

    def dump(self, data):
        from nicely import Printer
        Printer(**self.conf).print(data)


class TextDumper(Dumper):
    def dump(self, lines):
        for line in lines:
            print(line)




from caty.plugins import Plugins


def main(path, ext):
    handlers = {
        ext : Plugin()
        for Plugin in Plugins
        for ext in Plugin.ext
    }
    handler = handlers.get(ext,  handlers['hex'])
    try:
        data = handler.read(path)
        handler.dump(data)
    except Exception as x:
        print(x)



from pybrary.html_parser import HTML as Parser

from .base import RawReader, TextDumper


class HTML(RawReader, TextDumper):
    ext = 'html', 'htm'

    def load(self, raw):
        return Parser(raw)

    def parse(self, html):
        for node in html.body.walk_relative():
            yield node
            if node.data:
                yield node.data




from pathlib import Path
from sys import argv

from caty.main import main


def cli():
    ''' Dump the file given as arg.
    '''
    path = Path(argv[1])
    ext = argv[2] if len(argv)==3 else path.suffix[1:]
    main(path, ext)




from .base import FileReader, TextDumper


class Base(FileReader, TextDumper):
    mode = 'rb'

    def load_file(self, path):
        from binview import dumper as load
        return load(path, fmt=self.ext[0])

    def parse(self, lines):
        for line in lines:
            yield line


class BIN(Base):
    ext = 'bin',

class OCT(Base):
    ext = 'oct',

class DEC(Base):
    ext = 'dec',

class HEX(Base):
    ext = 'hex',



from .base import FileReader, DataDumper


class SQLITE(FileReader, DataDumper):
    ext = 'db', 'db3', 'sqlite'
    mode = 'rb'

    def load_file(self, path):
        from sqlview import DB
        return DB(path)

    def parse(self, db):
        return {
            table.name : table
            for table in db
        }



from operator import itemgetter

from .base import RawReader, DataDumper


class CFG(RawReader, DataDumper):
    ext = 'config', 'conf', 'cfg'

    def load(self, raw):
        return raw

    def parse(self, data):
        lines = [
            line
            for line in (
                d.strip()
                for d in data.split('\n')
            )
            if line
        ]
        for rem in ('#', "//", "'", '"'):
            lines = [
                line
                for line in lines
                if not line.startswith(rem)
            ]

        seps = {
            '=' : 0,
            ':' : 0,
            ' ' : 0,
        }
        for sep in seps:
            for i in (line.split(sep) for line in lines):
                if len(i)==2:
                    seps[sep]+=1

        sep = max(seps.items(), key=itemgetter(1))[0]

        config = dict(
            item
            for item in (
                line.split(sep, maxsplit=1)
                for line in lines
            )
            if len(item)==2
        )
        return config





from os.path import join, dirname, abspath

from setuptools import setup, find_packages

curdir = abspath(dirname(__file__))
readme = open(join(curdir, 'README.rst')).read()

setup(
    name             = 'caty',
    version          = '0.20.36.1',
    description      = 'Nice cat.',
    long_description = readme,
    keywords         = ['utility', ],
    url              = 'https://gitlab.com/dugres/caty/tree/stable',
    author           = 'Louis RIVIERE',
    author_email     = 'louis@riviere.xyz',
    license          = 'MIT',
    classifiers      = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    install_requires = [
        'nicely==0.2.0',
        'binview==0.1.2',
        'sqlview==0.2.0',
        'pybrary==0.20.36.1',
    ],
    package_dir = {
        'caty': 'caty',
    },
    packages = [
        'caty',
        'caty.plugins',
    ],
    entry_points = dict(
        console_scripts = (
            'caty=caty:cli',
        ),
    ),
)



from .base import RawReader, DataDumper


class INI(RawReader, DataDumper):
    ext = 'ini',

    def load(self, raw):
        from configparser import ConfigParser
        conf = ConfigParser()
        conf.read_string(raw)
        return conf

    def parse(self, conf):
        return {
            s : dict(conf.items(s))
            for s in conf.sections()
        }



from .base import FileReader, DataDumper


class CSV(FileReader, DataDumper):
    ext = 'csv',

    def load_file(self, path):
        from csv import DictReader
        with open(path) as inp:
            data = list(DictReader(inp))
        return data
